package com.github.gkutiel.fbi;

import static com.github.gkutiel.fbi.FbUtil.get;
import static com.github.gkutiel.fbi.FbUtil.url;
import static com.google.common.base.Joiner.on;
import static com.google.common.base.Preconditions.checkNotNull;

import java.util.HashMap;
import java.util.Map;

public class FbApp {
	public static class Client {
		public final String id;
		public final String secret;

		public Client(final String id, final String secret) {
			this.id = id;
			this.secret = secret;
		}
	}

	public static class FbLoginException extends Exception {

		private final String fbLoginPage;

		public FbLoginException(final String fbLoginPage) {
			this.fbLoginPage = fbLoginPage;
		}

		public String fbLoginPage() {
			return fbLoginPage;
		}

		@Override
		public String toString() {
			return "Login error, redirect to " + fbLoginPage;
		}
	}

	public static enum Permission {
		// @formatter:off
		PUBLISH_STREAM("publish_stream"), 
		EMAIL("email"), 
		
		FRIENDS_BIRTHDAY("friends_birthday"), 
		
		USER_BIRTHDAY("user_birthday"), 
		USER_EVENTS("user_events");
		//@formatter:on

		private String name;

		Permission(final String name) {
			this.name = name;
		}

		@Override
		public String toString() {
			return name;
		}
	}

	private final Client client;
	private final Permission[] permissions;

	public FbApp(final Client client, final Permission... permissions) {
		this.client = client;
		this.permissions = permissions;
	}

	public String fbLoginPage(final String callbackUrl) {
		final Map<String, String> args = new HashMap<>();
		args.put("client_id", client.id);
		args.put("redirect_uri", callbackUrl);
		args.put("scope", on(',').join(permissions));
		return FbUtil.url("https://www.facebook.com/dialog/oauth", args);
	}

	public String getAccessTokenFromFb(final String code, final String callbackUrl) throws FbLoginException {
		checkNotNull(code);
		try {
			final Map<String, String> args = new HashMap<>();
			args.put("client_id", client.id);
			args.put("client_secret", client.secret);
			args.put("redirect_uri", callbackUrl);
			args.put("code", code);
			final String response = get(url("https://graph.facebook.com/oauth/access_token", args));
			return extracteAccessToken(response);
		} catch (final Exception e) {
			throw new FbLoginException(fbLoginPage(callbackUrl));
		}
	}

	private String extracteAccessToken(final String response) {
		return response.replace("access_token=", "").replaceAll("&.*", "");
	}
}
