package com.github.gkutiel.fbi;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ExecutionException;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ning.http.client.AsyncHttpClient;

public class FbUtil {

	private static final AsyncHttpClient HTTP_CLIENT = new AsyncHttpClient();

	private static void appendArg(final StringBuilder queryString, final Entry<String, String> arg) {
		queryString.append(encode(arg.getKey()));
		queryString.append("=");
		queryString.append(encode(arg.getValue()));
	}

	private static String encode(final String val) {
		try {
			return URLEncoder.encode(val, "utf-8");
		} catch (final UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}

	static String get(final String url) {
		try {
			return HTTP_CLIENT.prepareGet(url).execute().get().getResponseBody();
		} catch (IOException | InterruptedException | ExecutionException e) {
			throw new RuntimeException(e);
		}
	}

	static Logger getLogger(final Class<?> c) {
		final Logger logger = Logger.getLogger(c.getName());
		final ConsoleHandler consoleHandler = new ConsoleHandler();
		consoleHandler.setLevel(Level.ALL);
		logger.addHandler(consoleHandler);
		return logger;
	}

	static String queryString(final Map<String, String> args) {
		final StringBuilder queryString = new StringBuilder();

		final ArrayList<Entry<String, String>> argsList = new ArrayList<>(args.entrySet());

		final Entry<String, String> firstArg = argsList.remove(0);
		appendArg(queryString, firstArg);

		for (final Entry<String, String> arg : argsList) {
			queryString.append("&");
			appendArg(queryString, arg);
		}

		return queryString.toString();
	}

	static String url(final String path, final Map<String, String> args) {
		return path + "?" + queryString(args);
	}

}
