package com.github.gkutiel.fbi;

import static com.github.gkutiel.fbi.FbUtil.get;
import static com.github.gkutiel.fbi.FbUtil.url;

import java.util.HashMap;
import java.util.Map;

import com.github.gkutiel.fbi.Fbi.FbiResponseException;

abstract class GraphAction<T> {
	static class PostGraphAction<T> extends GraphAction<T> {
		public PostGraphAction(final String accessToken, final String path, final Class<T> responseClass) {
			super(accessToken, path, responseClass);
			args.put("method", "post");
		}
	}

	private final String path;
	private final Class<T> responseClass;
	final Map<String, String> args = new HashMap<>();

	protected GraphAction(final String accessToken, final String path, final Class<T> responseClass) {
		args.put("access_token", accessToken);
		this.path = path;
		this.responseClass = responseClass;
	}

	public T go() throws FbiResponseException {
		final String url = url("https://graph.facebook.com/" + path, args);
		Fbi.LOGGER.fine("-->\t" + url);
		final String response = get(url);
		Fbi.LOGGER.fine("<--\t" + response);

		if (response.contains("\"error\"")) throw new FbiResponseException(response);

		return Fbi.GSON.fromJson(response, responseClass);
	}

	@SuppressWarnings("unchecked")
	<S> S param(final String key, final String val) {
		args.put(key, val);
		return (S) this;
	}
}